package de.intevation.flys.client.server;

import java.util.ArrayList;
import java.util.List;

import javax.xml.xpath.XPathConstants;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import de.intevation.artifacts.common.ArtifactNamespaceContext;
import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;

import de.intevation.flys.client.shared.model.DefaultRiver;
import de.intevation.flys.client.shared.model.River;
import de.intevation.flys.client.client.services.RiverService;


/**
 * This interface provides a method to list the supported rivers of the artifact
 * server.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class RiverServiceImpl
extends      RemoteServiceServlet
implements   RiverService
{
    /** The XPath string that points to the rivers in the resulting document.*/
    public static final String XPATH_RIVERS = "/art:rivers/art:river";


    public River[] list(String serverUrl) {
        Document doc      = XMLUtils.newDocument();

        XMLUtils.ElementCreator ec = new XMLUtils.ElementCreator(
            doc,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        doc.appendChild(ec.create("action"));

        HttpClient client = new HttpClientImpl(serverUrl);

        try {
            Document res = client.callService(serverUrl, "rivers", doc);

            NodeList rivers = (NodeList) XMLUtils.xpath(
                res,
                XPATH_RIVERS,
                XPathConstants.NODESET,
                ArtifactNamespaceContext.INSTANCE);

            int count = rivers.getLength();

            List<River> theRivers = new ArrayList<River>(count);

            for (int i = 0; i < count; i++) {
                Node tmp = rivers.item(i);

                String name = XMLUtils.xpathString(
                    tmp, "@art:name", ArtifactNamespaceContext.INSTANCE);

                theRivers.add(new DefaultRiver(name));
            }

            return (River[]) theRivers.toArray(new River[count]);
        }
        catch (ConnectionException ce) {
            System.err.println(ce.getLocalizedMessage());
        }

        return null;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
